/**
 * Provides the "DSL" (domain specific language) component of Razmin. This can be thought of as the UI,
 * providing developers the ability to tersely specify and run their tests.
 */
import { TestSuite } from "../suite";
import { SuiteSettings } from "../core";
import { TestSuiteResults } from "../suite";
import { TestSuiteFactory } from "./test-suite-factory";
import { TestSubjectBuilder } from "./test-subject-builder";
import { TestBuilder } from "./test-builder";
export declare class FluentSuite {
    constructor(settings?: SuiteSettings);
    private _settings;
    readonly suite: TestSuite;
    get settings(): SuiteSettings;
    withTimeout(millis: number): this;
    withOptions(settings: SuiteSettings): this;
    private suiteBuilders;
    include(paths: string[]): this;
    run(): Promise<TestSuiteResults>;
}
export declare function runSuite(paths: string[], options?: SuiteSettings): Promise<TestSuiteResults>;
export declare function buildSuite(paths: string[], options?: SuiteSettings, testSuite?: TestSuite): Promise<TestSuite>;
export declare const describe: TestSubjectBuilder;
export declare function skip(): void;
export declare function before(func: Function): void;
export declare function after(func: Function): void;
export declare const it: TestBuilder;
/**
 * Define and execute a test suite
 * @param builder
 */
export declare function suite(): FluentSuite;
export declare function suite(builder: TestSuiteFactory, settings?: SuiteSettings): Promise<TestSuiteResults>;
export declare function beforeEach(handler: Function): void;
export declare function afterEach(handler: Function): void;
