"use strict";
/**
 * Provides the "DSL" (domain specific language) component of Razmin. This can be thought of as the UI,
 * providing developers the ability to tersely specify and run their tests.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.afterEach = exports.beforeEach = exports.suite = exports.it = exports.after = exports.before = exports.skip = exports.describe = exports.buildSuite = exports.runSuite = exports.FluentSuite = void 0;
var suite_1 = require("../suite");
var core_1 = require("../core");
var subject_1 = require("../subject");
var test_1 = require("../test");
var callsites = require("callsites");
var path = require("path");
var util_1 = require("../util");
var reporting_1 = require("../reporting");
var test_reporting_settings_1 = require("../core/test-reporting-settings");
function applyDefaultSettings(settings) {
    if (!settings)
        settings = {};
    if (!settings.reporting)
        settings.reporting = {};
    if (!settings.reporting.reporters)
        settings.reporting.reporters = [new reporting_1.ConsoleReporter()];
    if (typeof settings.reporting.exitAndReport === 'undefined')
        settings.reporting.exitAndReport = true;
    return settings;
}
function include(moduleName) {
    try {
        return require(moduleName);
    }
    catch (e) {
        return null;
    }
}
var FluentSuite = /** @class */ (function () {
    function FluentSuite(settings) {
        if (settings === void 0) { settings = {}; }
        this.suiteBuilders = [];
        this._settings = applyDefaultSettings(settings);
        this.suite = new suite_1.TestSuite(new core_1.TestExecutionSettings(this._settings.execution), new test_reporting_settings_1.TestReportingSettings(this._settings.reporting));
    }
    Object.defineProperty(FluentSuite.prototype, "settings", {
        get: function () {
            return this._settings;
        },
        enumerable: false,
        configurable: true
    });
    FluentSuite.prototype.withTimeout = function (millis) {
        if (!this._settings.execution)
            this._settings.execution = {};
        this._settings.execution.timeout = millis;
        return this;
    };
    FluentSuite.prototype.withOptions = function (settings) {
        settings = applyDefaultSettings(settings);
        this._settings = settings;
        this.suite.executionSettings = new core_1.TestExecutionSettings(settings.execution);
        this.suite.reportingSettings = new test_reporting_settings_1.TestReportingSettings(settings.reporting);
        return this;
    };
    FluentSuite.prototype.include = function (paths) {
        var origDir = process.cwd();
        var callerFile = callsites()[1].getFileName();
        if (callerFile)
            process.chdir(path.dirname(callerFile));
        try {
            this.suiteBuilders.push(buildSuite(paths, this.settings, this.suite));
        }
        finally {
            process.chdir(origDir);
        }
        return this;
    };
    FluentSuite.prototype.run = function () {
        return __awaiter(this, void 0, void 0, function () {
            var e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, Promise.all(this.suiteBuilders)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        e_1 = _a.sent();
                        if (this.suite.reportingSettings.exitAndReport) {
                            console.error("Failed to build test suite:");
                            console.error(e_1);
                            process.exit(1);
                        }
                        else {
                            throw e_1;
                        }
                        return [3 /*break*/, 3];
                    case 3: return [4 /*yield*/, this.suite.run()];
                    case 4: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return FluentSuite;
}());
exports.FluentSuite = FluentSuite;
function runSuite(paths, options) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, buildSuite(paths, options)];
                case 1: return [4 /*yield*/, (_a.sent()).run()];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.runSuite = runSuite;
function buildSuite(paths, options, testSuite) {
    return __awaiter(this, void 0, void 0, function () {
        var top, zone;
        var _this = this;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    options = applyDefaultSettings(options);
                    top = false;
                    if (!testSuite) {
                        testSuite = new suite_1.TestSuite(new core_1.TestExecutionSettings(options.execution), new test_reporting_settings_1.TestReportingSettings(options.reporting));
                    }
                    zone = Zone.current.fork({
                        name: 'razminSuiteZone',
                        properties: {
                            razminTestSuite: testSuite,
                            razminLifecycleContainer: testSuite
                        },
                    });
                    return [4 /*yield*/, new Promise(function (resolve, reject) {
                            var requireGlob = include('require-glob');
                            zone.run(function () { return __awaiter(_this, void 0, void 0, function () {
                                return __generator(this, function (_a) {
                                    try {
                                        requireGlob.sync(paths, {
                                            cwd: process.cwd()
                                        });
                                        resolve();
                                    }
                                    catch (e) {
                                        console.error("Caught error while building test:");
                                        console.error(e);
                                        reject(e);
                                    }
                                    return [2 /*return*/];
                                });
                            }); });
                        })];
                case 1:
                    _a.sent();
                    return [2 /*return*/, testSuite];
            }
        });
    });
}
exports.buildSuite = buildSuite;
function describeRaw(description, testFactory, options) {
    return __awaiter(this, void 0, void 0, function () {
        var testSuite, parentSubject, parent, subject, zone;
        var _this = this;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    testSuite = suite_1.TestSuite.current;
                    if (!testSuite) {
                        suiteDeclaration(function () { return (0, exports.describe)(description, testFactory); });
                        return [2 /*return*/];
                    }
                    parentSubject = Zone.current.get('razminSubject');
                    if (parentSubject)
                        description = (0, util_1.descriptionConcat)(parentSubject.description, description);
                    parent = Zone.current.get('razminLifecycleContainer');
                    ;
                    if (!parent)
                        parent = testSuite;
                    subject = new subject_1.TestSubject(description, parent);
                    testSuite.addSubject(subject);
                    zone = Zone.current.fork({
                        name: 'razminSubjectZone',
                        properties: {
                            razminSubject: subject,
                            razminLifecycleContainer: subject
                        }
                    });
                    // TODO: use options
                    return [4 /*yield*/, new Promise(function (resolve, reject) {
                            zone.run(function () { return __awaiter(_this, void 0, void 0, function () {
                                var e_2;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            _a.trys.push([0, 2, , 3]);
                                            return [4 /*yield*/, testFactory(exports.it)];
                                        case 1:
                                            _a.sent();
                                            resolve();
                                            return [3 /*break*/, 3];
                                        case 2:
                                            e_2 = _a.sent();
                                            console.error("Caught error while building test:");
                                            console.error(e_2);
                                            reject(e_2);
                                            return [3 /*break*/, 3];
                                        case 3: return [2 /*return*/];
                                    }
                                });
                            }); });
                        })];
                case 1:
                    // TODO: use options
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
describeRaw['skip'] = function (desc, fac, opts) {
    describeRaw(desc, fac, Object.assign({ skip: true }, opts || {}));
};
describeRaw['only'] = function (desc, fac, opts) {
    describeRaw(desc, fac, Object.assign({ only: true }, opts || {}));
};
exports.describe = describeRaw;
function skip() {
    throw new test_1.Skip();
}
exports.skip = skip;
function before(func) {
    var subject = Zone.current.get('razminSubject');
    if (!subject)
        throw new Error("before() must be run within a describe() block");
    subject.addEventListener('before', function () { return func(); });
}
exports.before = before;
function after(func) {
    var subject = Zone.current.get('razminSubject');
    if (!subject)
        throw new Error("after() must be run within a describe() block");
    subject.addEventListener('after', function () { return func(); });
}
exports.after = after;
function itRaw(testDescription, func, options) {
    return __awaiter(this, void 0, void 0, function () {
        var subject;
        return __generator(this, function (_a) {
            subject = Zone.current.get('razminSubject');
            if (!subject)
                throw new Error("You can only call it() from inside a describe() block.");
            if (func) {
                subject.addTest(testDescription, func, options);
            }
            else if (!options) {
                subject.addTest(testDescription, function () { }, { skip: true });
            }
            else {
                subject.addTest(testDescription, function () {
                    throw new Error("You have passed an undefined test function to it(), "
                        + "but you included options! This is probably a mistake.");
                }, options);
            }
            return [2 /*return*/];
        });
    });
}
itRaw['skip'] = function (desc, func) { return (0, exports.it)(desc, func, { skip: true }); };
itRaw['only'] = function (desc, func) { return (0, exports.it)(desc, func, { only: true }); };
exports.it = itRaw;
function suiteDeclaration(builder, settings) {
    return __awaiter(this, void 0, void 0, function () {
        var testSuite, top, topLevelSuite, currentTest, zone, isolated, executionSettings, reportingSettings, finishedLoading, results, e_3;
        var _this = this;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    settings = applyDefaultSettings(settings);
                    top = false;
                    topLevelSuite = suite_1.TestSuite.current;
                    currentTest = Zone.current.get('razminTest');
                    zone = Zone.current;
                    isolated = settings && settings.execution && settings.execution.isolated;
                    // or if this suite() declaration lives within an it() declaration,
                    // then we must not join our results onto the parent suite
                    if (currentTest && topLevelSuite) {
                        if (topLevelSuite.executionSettings.verbose) {
                            console.warn("Warning: Detected a suite() declaration inside an it() test. "
                                + "The inner suite will run independently of the outer suite (forcing execution.isolated to be true)");
                        }
                        isolated = true;
                    }
                    // If we were asked to remain isolated from any parent suite, do so
                    if (isolated)
                        topLevelSuite = null;
                    if (topLevelSuite) {
                        testSuite = topLevelSuite;
                    }
                    else {
                        executionSettings = new core_1.TestExecutionSettings(settings.execution);
                        reportingSettings = new test_reporting_settings_1.TestReportingSettings(settings.reporting);
                        // If this new suite is being declared inside another Razmin test then 
                        // we must not use exitAndReport or the containing test will never complete,
                        // since we will run process.exit().
                        if (currentTest && reportingSettings.exitAndReport) {
                            if (executionSettings.verbose) {
                                console.warn("Warning: Suite declared inside of test case cannot use exitAndReport or outer suite will be interrupted. "
                                    + "(forcing reporting.exitAndReport to be false)");
                            }
                            reportingSettings.exitAndReport = false;
                        }
                        testSuite = topLevelSuite = new suite_1.TestSuite(executionSettings, reportingSettings);
                        top = true;
                        zone = zone.fork({
                            name: 'razminSuiteZone',
                            properties: {
                                razminTestSuite: testSuite,
                                // When you have a suite() inside it(), we want that suite 
                                // to not join the parent suite, but we DO want subsuites inside
                                // the inner suite to join to THAT, so clear razminTest so we 
                                // don't trigger the isolation behavior further down the stack
                                razminTest: null
                            }
                        });
                    }
                    finishedLoading = testSuite._startLoading();
                    return [4 /*yield*/, zone.run(function () { return __awaiter(_this, void 0, void 0, function () {
                            var _this = this;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, new Promise(function (resolve, reject) {
                                            // Construct a new zone to track async activity in the suite
                                            var testZone = new util_1.TestZone('razminSuiteTracker');
                                            testZone.onError.subscribe(function (err) { return reject(err); });
                                            testZone.onStable.subscribe(function () { return resolve(); });
                                            testZone.zone.run(function () { return __awaiter(_this, void 0, void 0, function () {
                                                var e_4;
                                                return __generator(this, function (_a) {
                                                    switch (_a.label) {
                                                        case 0:
                                                            _a.trys.push([0, 2, , 3]);
                                                            return [4 /*yield*/, builder(exports.describe)];
                                                        case 1:
                                                            _a.sent();
                                                            return [3 /*break*/, 3];
                                                        case 2:
                                                            e_4 = _a.sent();
                                                            console.error("Caught error while building test:");
                                                            console.error(e_4);
                                                            reject(e_4);
                                                            return [3 /*break*/, 3];
                                                        case 3: return [2 /*return*/];
                                                    }
                                                });
                                            }); });
                                        })];
                                    case 1:
                                        _a.sent();
                                        return [2 /*return*/];
                                }
                            });
                        }); })];
                case 1:
                    _a.sent();
                    finishedLoading();
                    if (!top)
                        return [2 /*return*/];
                    topLevelSuite = null;
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, testSuite.run()];
                case 3:
                    results = _a.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_3 = _a.sent();
                    console.error("Caught error while running test suite:");
                    console.error(e_3);
                    throw e_3;
                case 5: return [2 /*return*/, results];
            }
        });
    });
}
function argsMatch(args, patterns) {
    var satisfiedArgs = [];
    var mustBeOptional = false;
    var _loop_1 = function (i, max) {
        var value = args[i];
        var hasValue = args.length > i;
        var pattern = patterns[i];
        var optional = false;
        if (hasValue && !pattern)
            return { value: false };
        if (pattern.endsWith('?')) {
            pattern = pattern.replace(/\?$/g, '');
            optional = true;
            mustBeOptional = true;
        }
        else if (mustBeOptional) {
            throw new Error("A non-optional descriptor cannot follow an optional one");
        }
        if (optional && value === undefined)
            return { value: true };
        if (!hasValue) {
            if (optional)
                return { value: true };
            else
                return { value: false };
        }
        var matchers = {
            null: function (v) { return v === null; },
            undefined: function (v) { return v === null; },
            object: function (v) { return typeof v === 'object'; },
            function: function (v) { return typeof v === 'function'; },
            number: function (v) { return typeof v === 'number'; },
            boolean: function (v) { return typeof v === 'boolean'; },
            string: function (v) { return typeof v === 'string'; },
            true: function (v) { return v === true; },
            false: function (v) { return v === false; }
        };
        var matches = pattern.split(/\|/g).map(function (pattern) {
            if (!matchers[pattern])
                throw new Error("Unsupported pattern '".concat(pattern, "'"));
            var matcher = matchers[pattern];
            var matches = matcher(value);
            return matches;
        }).find(function (x) { return x; });
        if (!matches)
            return { value: false };
        satisfiedArgs.push(value);
    };
    for (var i = 0, max = Math.max(args.length, patterns.length); i < max; ++i) {
        var state_1 = _loop_1(i, max);
        if (typeof state_1 === "object")
            return state_1.value;
    }
    if (satisfiedArgs.length !== patterns.length)
        return false;
    return true;
}
function suite() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    if (argsMatch(args, ['object?']))
        return new FluentSuite(args[0]);
    if (argsMatch(args, ['function', 'object?']))
        return suiteDeclaration(args[0], args[1]);
    throw new Error("Unknown parameters: ".concat(args.join(', ')));
}
exports.suite = suite;
function beforeEach(handler) {
    var lifecycleContainer = Zone.current.get('razminLifecycleContainer');
    if (!lifecycleContainer)
        throw new Error("Can only be used inside suite() or describe()");
    lifecycleContainer.addEventListener('before', handler);
}
exports.beforeEach = beforeEach;
function afterEach(handler) {
    var lifecycleContainer = Zone.current.get('razminLifecycleContainer');
    if (!lifecycleContainer)
        throw new Error("Can only be used inside suite() or describe()");
    lifecycleContainer.addEventListener('after', handler);
}
exports.afterEach = afterEach;
//# sourceMappingURL=dsl.js.map