"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsoleDotsReporter = void 0;
var colors = require("colors/safe");
var util_1 = require("../util");
var ConsoleDotsReporter = /** @class */ (function () {
    function ConsoleDotsReporter() {
    }
    ConsoleDotsReporter.prototype.onTestFinished = function (suite, subject, test, result) {
        if (typeof process !== 'undefined') {
            if (result.passed === 'skip') {
                process.stdout.write(colors.yellow("S"));
            }
            else if (result.passed) {
                process.stdout.write(colors.green("."));
            }
            else {
                process.stdout.write(colors.red("F"));
            }
        }
    };
    ConsoleDotsReporter.prototype.onSuiteFinished = function (suite, results) {
        console.log();
        var total = 0;
        var passed = 0;
        var failed = 0;
        var skipped = 0;
        for (var _i = 0, _a = results.subjectResults; _i < _a.length; _i++) {
            var subjectResult = _a[_i];
            for (var _b = 0, _c = subjectResult.tests; _b < _c.length; _b++) {
                var test = _c[_b];
                total += 1;
                if (test.passed === 'skip')
                    skipped += 1;
                else if (test.passed)
                    passed += 1;
                else
                    failed += 1;
            }
        }
        for (var _d = 0, _e = results.subjectResults; _d < _e.length; _d++) {
            var subjectResult = _e[_d];
            if (subjectResult.tests.length == 0)
                continue;
            if (!subjectResult.tests.find(function (x) { return !x.passed; }))
                continue;
            console.log();
            console.log(colors.yellow(colors.underline(subjectResult.description)));
            for (var _f = 0, _g = subjectResult.tests; _f < _g.length; _f++) {
                var testResult = _g[_f];
                if (testResult.passed)
                    continue;
                var report = !testResult.hidden;
                var blankSpot = '     ';
                var indicator = blankSpot;
                var color = function (c) { return c; };
                var printMessage = false;
                var durationText = "";
                var slowThreshold = results.testSuite.reportingSettings.slowThreshold;
                var minimumDuration = results.testSuite.reportingSettings.minimumReportedDuration;
                var fast = testResult.duration < minimumDuration;
                var slow = testResult.duration > 0.5 * slowThreshold;
                var verySlow = testResult.duration > slowThreshold;
                if (verySlow)
                    durationText = " ".concat(colors.red(colors.bold("[".concat(testResult.duration, "ms]"))));
                else if (slow)
                    durationText = " ".concat(colors.yellow(colors.bold("[".concat(testResult.duration, "ms]"))));
                else if (!fast)
                    durationText = " ".concat(colors.cyan("[".concat(testResult.duration, "ms]")));
                if (testResult.passed === 'skip') {
                    skipped += 1;
                    color = colors.yellow;
                    indicator = ' (S) ';
                }
                else if (testResult.passed) {
                    passed += 1;
                    color = colors.green;
                    indicator = '  ✓  ';
                }
                else {
                    failed += 1;
                    color = colors.red;
                    indicator = '  ✗  ';
                    printMessage = true;
                }
                if (report) {
                    console.log(color("".concat(indicator).concat((0, util_1.descriptionConcat)(subjectResult.description, testResult.description)).concat(durationText)));
                    if (printMessage)
                        console.log("".concat(blankSpot).concat(testResult.message));
                }
            }
        }
        console.log();
        var allPassed = results.subjectResults.map(function (x) { return x.passed; }).indexOf(false) < 0;
        if ((failed > 0 && passed > 0) || skipped > 0)
            console.log("".concat(total, " tests: ").concat(colors.green("".concat(passed, " passed")), ", ").concat(colors.yellow("".concat(skipped, " skipped")), ", ").concat(colors.red("".concat(failed, " failed"))));
        else if (failed > 0)
            console.log("".concat(colors.red("".concat(failed, " / ").concat(total, " test(s) failed"))));
        else if (total > 0)
            console.log("".concat(colors.green("".concat(passed, " test(s) passed"))));
        else
            console.log("".concat(colors.yellow("0 test(s) defined")));
        console.log();
    };
    return ConsoleDotsReporter;
}());
exports.ConsoleDotsReporter = ConsoleDotsReporter;
//# sourceMappingURL=console-dots-reporter.js.map