"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsoleReporter = void 0;
var colors = require("colors/safe");
var util_1 = require("../util");
var ConsoleReporter = /** @class */ (function () {
    function ConsoleReporter() {
    }
    ConsoleReporter.prototype.onSuiteFinished = function (suite, results) {
        var total = 0;
        var passed = 0;
        var failed = 0;
        var skipped = 0;
        for (var _i = 0, _a = results.subjectResults; _i < _a.length; _i++) {
            var subjectResult = _a[_i];
            var visibleCount = subjectResult.tests.filter(function (x) { return x.hidden !== true; }).length;
            if (visibleCount > 0) {
                console.log();
                console.log(colors.yellow(colors.underline(subjectResult.description)));
            }
            for (var _b = 0, _c = subjectResult.tests; _b < _c.length; _b++) {
                var testResult = _c[_b];
                total += 1;
                var report = !testResult.hidden;
                var blankSpot = '     ';
                var indicator = blankSpot;
                var color = function (c) { return c; };
                var printMessage = false;
                var durationText = "";
                var slowThreshold = results.testSuite.reportingSettings.slowThreshold;
                var minimumDuration = results.testSuite.reportingSettings.minimumReportedDuration;
                var fast = testResult.duration < minimumDuration;
                var slow = testResult.duration > 0.5 * slowThreshold;
                var verySlow = testResult.duration > slowThreshold;
                if (verySlow)
                    durationText = " ".concat(colors.red(colors.bold("[".concat(testResult.duration, "ms]"))));
                else if (slow)
                    durationText = " ".concat(colors.yellow(colors.bold("[".concat(testResult.duration, "ms]"))));
                else if (!fast)
                    durationText = " ".concat(colors.cyan("[".concat(testResult.duration, "ms]")));
                if (testResult.passed === 'skip') {
                    skipped += 1;
                    color = colors.yellow;
                    indicator = ' (S) ';
                }
                else if (testResult.passed) {
                    passed += 1;
                    color = colors.green;
                    indicator = '  ✓  ';
                }
                else {
                    failed += 1;
                    color = colors.red;
                    indicator = '  ✗  ';
                    printMessage = true;
                }
                if (report) {
                    console.log(color("".concat(indicator).concat((0, util_1.descriptionConcat)(subjectResult.description, testResult.description)).concat(durationText)));
                    if (printMessage)
                        console.log("".concat(blankSpot).concat(testResult.message));
                }
            }
        }
        console.log();
        var allPassed = results.subjectResults.map(function (x) { return x.passed; }).indexOf(false) < 0;
        if ((failed > 0 && passed > 0) || skipped > 0) {
            var parts = [];
            if (passed > 0)
                parts.push("".concat(colors.green("".concat(passed, " passed"))));
            if (skipped > 0)
                parts.push("".concat(colors.yellow("".concat(skipped, " skipped"))));
            if (failed > 0)
                parts.push("".concat(colors.red("".concat(failed, " failed"))));
            console.log("ran ".concat(total, " test(s): ").concat(parts.join(', ')));
        }
        else if (failed > 0) {
            console.log("".concat(colors.red("".concat(failed, " / ").concat(total, " test(s) failed"))));
        }
        else if (total > 0) {
            console.log("".concat(colors.green("".concat(passed, " test(s) passed"))));
        }
        else {
            console.log("".concat(colors.yellow("0 tests defined")));
        }
        console.log();
    };
    return ConsoleReporter;
}());
exports.ConsoleReporter = ConsoleReporter;
//# sourceMappingURL=console-reporter.js.map