import { Test, TestOptions } from "../test";
import { TestFunction } from "../test";
import { TestSubjectResult } from "../subject";
import { TestExecutionSettings } from "../core";
import { LifecycleContainer } from "../util";
import { Reporter } from "../reporting";
/**
 * Represents a set of unit tests built around a single object under test ("subject")
 */
export declare class TestSubject implements LifecycleContainer {
    private _description;
    private _parent?;
    constructor(_description: string, _parent?: LifecycleContainer);
    private _tests;
    private _lifecycleEvents;
    addEventListener(eventName: string, handler: Function): void;
    fireEvent(eventName: string): Promise<void>;
    /**
     * Get the human-readable description of this subject.
     */
    get description(): string;
    /**
     * Get the set of defined tests for this subject.
     */
    get tests(): Test[];
    /**
     * Add a test which pertains to this test subject
     * @param description
     * @param func
     */
    addTest(description: string, func: TestFunction, options?: TestOptions): void;
    /**
     * Run an individual test with the given execution settings and an appropriate
     * context name.
     *
     * @param test
     * @param testExecutionSettings
     */
    private runTest;
    /**
     * Run all tests that deal with this test subject.
     * @param testExecutionSettings
     */
    run(reporters?: Reporter[], testExecutionSettings?: TestExecutionSettings): Promise<TestSubjectResult>;
}
