"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestSubject = void 0;
var test_1 = require("../test");
var subject_1 = require("../subject");
var test_2 = require("../test");
var util_1 = require("../util");
var MersenneTwister = require("mersenne-twister");
/**
 * Represents a set of unit tests built around a single object under test ("subject")
 */
var TestSubject = /** @class */ (function () {
    function TestSubject(_description, _parent) {
        this._description = _description;
        this._parent = _parent;
        this._tests = [];
        this._lifecycleEvents = {};
    }
    TestSubject.prototype.addEventListener = function (eventName, handler) {
        if (!this._lifecycleEvents[eventName])
            this._lifecycleEvents[eventName] = [];
        this._lifecycleEvents[eventName].push(handler);
    };
    TestSubject.prototype.fireEvent = function (eventName) {
        return __awaiter(this, void 0, void 0, function () {
            var handlers, _i, handlers_1, handler;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this._parent) return [3 /*break*/, 2];
                        return [4 /*yield*/, this._parent.fireEvent(eventName)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        handlers = this._lifecycleEvents[eventName] || [];
                        _i = 0, handlers_1 = handlers;
                        _a.label = 3;
                    case 3:
                        if (!(_i < handlers_1.length)) return [3 /*break*/, 6];
                        handler = handlers_1[_i];
                        return [4 /*yield*/, handler()];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        _i++;
                        return [3 /*break*/, 3];
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    Object.defineProperty(TestSubject.prototype, "description", {
        /**
         * Get the human-readable description of this subject.
         */
        get: function () {
            return this._description;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TestSubject.prototype, "tests", {
        /**
         * Get the set of defined tests for this subject.
         */
        get: function () {
            return this._tests;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Add a test which pertains to this test subject
     * @param description
     * @param func
     */
    TestSubject.prototype.addTest = function (description, func, options) {
        this._tests.push(new test_1.Test(description, func, options));
    };
    /**
     * Run an individual test with the given execution settings and an appropriate
     * context name.
     *
     * @param test
     * @param testExecutionSettings
     */
    TestSubject.prototype.runTest = function (test, testExecutionSettings) {
        return test.run(testExecutionSettings, "".concat((0, util_1.descriptionConcat)(this._description, test.description)));
    };
    /**
     * Run all tests that deal with this test subject.
     * @param testExecutionSettings
     */
    TestSubject.prototype.run = function (reporters, testExecutionSettings) {
        return __awaiter(this, void 0, void 0, function () {
            var results, tests, only, _i, tests_1, test, order, mt_1, seed, onlyMode, _a, tests_2, test, result, _b, reporters_1, reporter, _c, reporters_2, reporter;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        results = [];
                        tests = this._tests;
                        only = [];
                        if (!reporters)
                            reporters = [];
                        for (_i = 0, tests_1 = tests; _i < tests_1.length; _i++) {
                            test = tests_1[_i];
                            if (test.options.only)
                                only.push(test);
                        }
                        order = 'default';
                        if (testExecutionSettings && testExecutionSettings.order && testExecutionSettings.order != '')
                            order = testExecutionSettings.order;
                        if (order == 'random') {
                            mt_1 = new MersenneTwister();
                            seed = testExecutionSettings ? testExecutionSettings.orderSeed : undefined;
                            if (typeof seed === 'undefined')
                                seed = Math.round(Math.random() * 100000);
                            console.log("Ordering seed: ".concat(seed));
                            mt_1.init_seed(seed);
                            tests = tests.sort(function (a, b) { return mt_1.random() - 0.5; });
                        }
                        else if (order == 'default') {
                            // order as observed
                        }
                        else {
                            throw new Error("Test ordering '".concat(testExecutionSettings.order, "' is not supported"));
                        }
                        onlyMode = testExecutionSettings && testExecutionSettings.only;
                        _a = 0, tests_2 = tests;
                        _d.label = 1;
                    case 1:
                        if (!(_a < tests_2.length)) return [3 /*break*/, 6];
                        test = tests_2[_a];
                        result = void 0;
                        if (onlyMode && !only.includes(test)) {
                            result = new test_2.TestResult({
                                description: test.description,
                                passed: 'skip',
                                message: 'Skipped',
                                hidden: true
                            });
                            results.push(result);
                            return [3 /*break*/, 5];
                        }
                        for (_b = 0, reporters_1 = reporters; _b < reporters_1.length; _b++) {
                            reporter = reporters_1[_b];
                            if (reporter.onTestStarted)
                                reporter.onTestStarted(this._parent, this, test);
                        }
                        return [4 /*yield*/, this.fireEvent('before')];
                    case 2:
                        _d.sent();
                        return [4 /*yield*/, this.runTest(test, testExecutionSettings)];
                    case 3:
                        result = _d.sent();
                        return [4 /*yield*/, this.fireEvent('after')];
                    case 4:
                        _d.sent();
                        for (_c = 0, reporters_2 = reporters; _c < reporters_2.length; _c++) {
                            reporter = reporters_2[_c];
                            if (reporter.onTestFinished)
                                reporter.onTestFinished(this._parent, this, test, result);
                        }
                        results.push(result);
                        _d.label = 5;
                    case 5:
                        _a++;
                        return [3 /*break*/, 1];
                    case 6: return [2 /*return*/, new subject_1.TestSubjectResult(this._description, results)];
                }
            });
        });
    };
    return TestSubject;
}());
exports.TestSubject = TestSubject;
//# sourceMappingURL=test-subject.js.map