"use strict";
/**
 * Copyright Google Inc. All Rights Reserved.
 * Copyright William Lahti
 *
 * Derived from Angular's NgZone
 * https://github.com/angular/angular/blob/6.0.0/packages/core/src/zone/ng_zone.ts
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestZone = void 0;
var rxjs_1 = require("rxjs");
var TestZone = /** @class */ (function () {
    function TestZone(name, properties) {
        if (properties === void 0) { properties = {}; }
        this._zonesWithPendingMicrotasks = [];
        this._zonesWithPendingMacrotasks = [];
        this._nesting = 0;
        this._hasPendingMicrotasks = false;
        this._hasPendingMacrotasks = false;
        this._onError = new rxjs_1.Subject();
        this._onMicrotaskEmpty = new rxjs_1.Subject();
        this._onStable = new rxjs_1.Subject();
        var self = this;
        this._outside = Zone.current;
        this._zone = Zone.current.fork({
            name: name,
            properties: Object.assign({
                isTestCoreZone: true
            }, properties),
            onInvoke: function (delegate, current, target, callback, applyThis, applyArgs, source) {
                return self.innerInvoke(function () { return delegate.invoke(target, callback, applyThis, applyArgs, source); });
            },
            onInvokeTask: function (delegate, current, target, task, applyThis, applyArgs) {
                return self.innerInvoke(function () { return delegate.invokeTask(target, task, applyThis, applyArgs); });
            },
            onCancelTask: function (pz, cz, tz, task) {
                pz.cancelTask(tz, task);
            },
            onHasTask: function (delegate, current, target, hasTaskState) {
                //console.log(`onHasTask: ${JSON.stringify(hasTaskState)}`);
                delegate.hasTask(target, hasTaskState);
                if (hasTaskState.change == 'microTask') {
                    if (hasTaskState.microTask) {
                        if (!self._zonesWithPendingMicrotasks.includes(target))
                            self._zonesWithPendingMicrotasks.push(target);
                    }
                    else {
                        self._zonesWithPendingMicrotasks = self._zonesWithPendingMicrotasks.filter(function (x) { return x !== target; });
                    }
                    self._hasPendingMicrotasks = hasTaskState.microTask;
                }
                else if (hasTaskState.change == 'macroTask') {
                    self._hasPendingMacrotasks = hasTaskState.macroTask;
                    if (hasTaskState.macroTask) {
                        if (!self._zonesWithPendingMacrotasks.includes(target))
                            self._zonesWithPendingMacrotasks.push(target);
                    }
                    else {
                        self._zonesWithPendingMacrotasks = self._zonesWithPendingMacrotasks.filter(function (x) { return x !== target; });
                    }
                }
                self.checkStable();
            },
            onHandleError: function (delegate, current, target, error) {
                self.runOutside(function () { return self._onError.next(error); });
                return false;
            }
        });
    }
    Object.defineProperty(TestZone.prototype, "zone", {
        get: function () {
            return this._zone;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TestZone.prototype, "onMicrotaskEmpty", {
        get: function () {
            return this._onMicrotaskEmpty;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TestZone.prototype, "onStable", {
        get: function () {
            return this._onStable;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TestZone.prototype, "onError", {
        get: function () {
            return this._onError;
        },
        enumerable: false,
        configurable: true
    });
    TestZone.prototype.checkStable = function () {
        //console.log(`checkStable: nesting=${this._nesting}, macro=${this._hasPendingMacrotasks}, micro=${this._hasPendingMicrotasks}, stable=${this._isStable}`);
        var _this = this;
        if (this._nesting > 0 || this._hasPendingMacrotasks || this._hasPendingMicrotasks || this._isStable)
            return;
        if (this._zonesWithPendingMicrotasks.length > 0 || this._zonesWithPendingMacrotasks.length > 0)
            return;
        try {
            this._nesting++;
            this._onMicrotaskEmpty.next();
        }
        finally {
            this._nesting--;
        }
        if (this._hasPendingMicrotasks)
            return;
        // stable 
        //console.log(`stable!!`);
        try {
            this.runOutside(function () { return _this._onStable.next(null); });
        }
        finally {
            this._isStable = true;
        }
    };
    TestZone.prototype.runOutside = function (runnable) {
        this._outside.run(runnable);
    };
    TestZone.prototype.invoke = function (callback) {
        var _this = this;
        this._zone.runGuarded(function () { return _this.innerInvoke(callback); });
    };
    TestZone.prototype.innerInvoke = function (callback) {
        this.onEnter();
        var caughtError = false;
        try {
            return callback();
        }
        catch (e) {
            caughtError = true;
            throw e;
        }
        finally {
            this.onLeave();
            if (!caughtError)
                this.checkStable();
        }
    };
    TestZone.prototype.onEnter = function () {
        //console.log(`onEnter: depth ${this._nesting} -> ${this._nesting + 1}`);
        this._nesting += 1;
    };
    TestZone.prototype.onLeave = function () {
        //console.log(`onLeave: depth ${this._nesting} -> ${this._nesting - 1}`);
        this._nesting -= 1;
    };
    return TestZone;
}());
exports.TestZone = TestZone;
//# sourceMappingURL=test-zone.js.map